/***************************************
	Auteur : Pierre Aubert
	Mail : pierre.aubert@lapp.in2p3.fr
	Licence : CeCILL-C
****************************************/

#ifndef __ALIGNED_ALLOCATOR_H__
#define __ALIGNED_ALLOCATOR_H__

#include <new>
#include <limits>
#include <iostream>
#include <vector>

#include <malloc.h>

///@brief Allocator wihch alignes data
template<typename T>
struct AlignedAllocator{
	///Value of the type of the current allocator (used by std::vector to avoid ambiguities if we use multiple template)
	typedef T value_type;
	AlignedAllocator () = default;	//Declare the current constructor as default constructor
	
	///Copy constructor from U type
	template<typename U>
	constexpr AlignedAllocator (const AlignedAllocator <U>&) noexcept {}
	
	///Do the memory allocation
	/**	@param n : number of element to be allocated
	 * 	@return pointer to the allocated memory
	 * 	This function is nodiscard, so return value cannot be ignored
	 * 	We have to remove nodiscard when we make it static (and it works with std::vector)
	*/
	static /*[[nodiscard]]*/ T* allocate(std::size_t n){
		if(n > std::numeric_limits<std::size_t>::max() / sizeof(T)){
			throw std::bad_array_new_length();
		}
		T* p = static_cast<T*>(std::aligned_alloc(VECTOR_ALIGNEMENT, n * sizeof(T)));
		if(p != NULL){
			return p;
		}
		throw std::bad_alloc();
	}
	///Free the allocated memory
	/**	@param p : pointer to be freed
	 * 	@param n : number of element allocated in p
	*/
	static void deallocate(T* p, std::size_t n) noexcept{
		std::free(p);
	}
};


#endif


