/***************************************
	Auteur : Pierre Aubert
	Mail : pierre.aubert@lapp.in2p3.fr
	Licence : CeCILL-C
****************************************/
#include <iostream>
#include <vector>
#include <numeric>

#include <execution>
#include <algorithm>

#include "micro_benchmark.h"

///Defines a vector of index
typedef std::vector<int> VecIndex;

///Compute the calibration
/**	@param[out] vecCalibSignal : vector of calibrated signal
 * 	@param tabADC : pointer to the table of ADC values
 * 	@param tabPed : pointer to the table of pedestal
 * 	@param tabGain : pointer to the table of gain
 * 	@param vecEventIdx : vector of index of events
 * 	@param vecSliceIdx : vector of index of slices
 * 	@param vecPixelIdx : vector of index of pixels
*/
void compute_calibration(float * vecCalibSignal,
			const float * tabADC, const float * tabPed, const float* tabGain,
			const VecIndex & vecEventIdx, const VecIndex & vecSliceIdx, const VecIndex & vecPixelIdx)
{
	size_t nbSlice(vecSliceIdx.size()), nbPixel(vecPixelIdx.size());
	size_t eventSize(nbSlice*nbPixel);
	std::for_each(std::execution::seq, std::begin(vecEventIdx), std::end(vecEventIdx),
		[=](int evtIdx){
			std::for_each(std::execution::seq, std::begin(vecSliceIdx), std::end(vecSliceIdx),
				[=](int sliceIdx){
					std::for_each(EXECUTION_POLICY, std::begin(vecPixelIdx), std::end(vecPixelIdx),
						[=](int pixelIdx){
							long currentEventSlicePixelIdx = evtIdx*eventSize + sliceIdx*nbPixel + pixelIdx;
							vecCalibSignal[currentEventSlicePixelIdx] = (tabADC[currentEventSlicePixelIdx] - tabPed[pixelIdx])*tabGain[pixelIdx];
						}
					);
				}
			);
		}
	);
}

///Get the number of nanoseconds per elements of the Calibration
/**	@param nbPixel : number of pixels of the tables
*/
void evaluateCalibration(size_t nbPixel){
	//Let's define size of data :
	size_t nbEvent(NB_EVENT), nbSlice(NB_SLICE);
	size_t nbValuePerImage(nbSlice*nbPixel);
	size_t nbElement(nbEvent*nbValuePerImage);
	std::vector<float> vecGain(nbPixel), vecPedestal(nbPixel);
	std::fill(vecGain.begin(), vecGain.end(), 0.02f);
	std::fill(vecPedestal.begin(), vecPedestal.end(), 40.0f);
	
	std::vector<float> vecADCSignal(nbElement), vecCalibSignal(nbElement);
	std::fill(vecADCSignal.begin(), vecADCSignal.end(), 42.0f);
	
	VecIndex vecEventIdx(nbEvent), vecSliceIdx(nbSlice), vecPixelIdx(nbPixel);	//Init vectors of index for the computation
	std::iota(vecEventIdx.begin(), vecEventIdx.end(), 0);
	std::iota(vecSliceIdx.begin(), vecSliceIdx.end(), 0);
	std::iota(vecPixelIdx.begin(), vecPixelIdx.end(), 0);
	
	//We have to create pointer to be able to catch them by copy without losing any time
	float * tabADC = vecADCSignal.data(), *tabGain = vecGain.data(), *tabPed = vecPedestal.data(), *tabCalibSignal = vecCalibSignal.data();
	
	size_t fullNbElement(nbElement);
	micro_benchmarkAutoNsPrint("evaluateCalibration", fullNbElement, compute_calibration, tabCalibSignal, tabADC, tabPed, tabGain, vecEventIdx, vecSliceIdx, vecPixelIdx);
}

int main(int argc, char** argv){
	return micro_benchmarkParseArg(argc, argv, evaluateCalibration);
}
