/***************************************
	Auteur : Pierre Aubert
	Mail : pierre.aubert@lapp.in2p3.fr
	Licence : CeCILL-C
****************************************/
#include <iostream>
#include <vector>
#include <numeric>

#include <execution>
#include <algorithm>

#include "micro_benchmark.h"

#include "AlignedAllocator.h"

///Defines a vector of index
typedef std::vector<int> VecIndex;

///Defines a vector of data
typedef std::vector<float, AlignedAllocator<float> > VecData;

///Compute the calibration
/**	@param[out] tabCalibSignal : table of calibrated signal (input tabADC)
 * 	@param tabPed : pointer to the table of pedestal
 * 	@param tabGain : pointer to the table of gain
 * 	@param vecEventIdx : vector of index of events
 * 	@param nbPixel : number of pixels
*/
void compute_calibration(float * tabCalibSignal, const float * tabPed, const float* tabGain,
			const VecIndex & vecEventIdx, size_t nbPixel)
{
	std::for_each(std::execution::seq, std::begin(vecEventIdx), std::end(vecEventIdx),
		[=](int evtIdx){
			long firstPixelIdx = evtIdx*nbPixel;
			std::transform(EXECUTION_POLICY, tabCalibSignal + firstPixelIdx, tabCalibSignal + firstPixelIdx + nbPixel, tabPed, tabCalibSignal + firstPixelIdx,
				[=](float signal, float ped){
					return signal - ped;
				}
			);
			std::transform(EXECUTION_POLICY, tabCalibSignal + firstPixelIdx, tabCalibSignal + firstPixelIdx + nbPixel, tabGain, tabCalibSignal + firstPixelIdx,
				[=](float signal, float gain){
					return signal * gain;
				}
			);
		}
	);
}

///Get the number of nanoseconds per elements of the Calibration
/**	@param nbPixel : number of pixels of the tables
*/
void evaluateCalibration(size_t nbPixel){
	//Let's define size of data :
	size_t nbEvent(NB_EVENT*NB_SLICE);
	size_t nbValuePerImage(nbPixel);
	size_t nbElement(nbEvent*nbValuePerImage);
	VecData vecGain(nbPixel), vecPedestal(nbPixel);
	std::fill(vecGain.begin(), vecGain.end(), 0.02f);
	std::fill(vecPedestal.begin(), vecPedestal.end(), 40.0f);
	
	VecData vecADCSignal(nbElement);
	std::fill(vecADCSignal.begin(), vecADCSignal.end(), 42.0f);
	
	VecIndex vecEventIdx(nbEvent);	//Init vectors of index for the computation
	std::iota(vecEventIdx.begin(), vecEventIdx.end(), 0);
	
	//We have to create pointer to be able to catch them by copy without losing any time
	float * tabADC = vecADCSignal.data(), *tabGain = vecGain.data(), *tabPed = vecPedestal.data();
	
	size_t fullNbElement(nbElement);
	micro_benchmarkAutoNsPrint("evaluateCalibration 2d tranform", fullNbElement, compute_calibration, tabADC, tabPed, tabGain, vecEventIdx, nbPixel);
}

int main(int argc, char** argv){
	return micro_benchmarkParseArg(argc, argv, evaluateCalibration);
}
