/***************************************
	Auteur : Pierre Aubert
	Mail : pierre.aubert@lapp.in2p3.fr
	Licence : CeCILL-C
****************************************/
#include <iostream>
#include <vector>
#include <numeric>

#include <ranges>
#include <execution>
#include <algorithm>

#include "micro_benchmark.h"

///Defines a vector of index
typedef std::vector<int> VecIndex;

///Compute the calibration
/**	@param[out] tabCalibSignal : table of calibrated signal
 * 	@param tabADC : pointer to the table of ADC values
 * 	@param vecPedestal : vectpr of pedestal
 * 	@param vecGain : vector of gain
 * 	@param vecEventIdx : vector of index of events
 * 	@param vecPixelIdx : vector of index of pixels
*/
void compute_calibration(float * tabCalibSignal,
			const float * tabADC, const std::vector<float> & vecPedestal, const std::vector<float> & vecGain,
			const VecIndex & vecEventIdx, const VecIndex & vecPixelIdx)
{
	size_t nbPixel(vecPixelIdx.size());
	std::for_each(std::execution::seq, std::begin(vecEventIdx), std::end(vecEventIdx),
		[=](int evtIdx){
			long firstPixelIdx = evtIdx*nbPixel;
			std::transform(EXECUTION_POLICY, tabADC + firstPixelIdx, tabADC + firstPixelIdx + nbPixel, std::views::zip(vecPedestal, vecGain), tabCalibSignal,
				[=](float signal, std::tuple<float, float> pedGain){
					return (std::get<0>(pedGain) - signal) * std::get<1>(pedGain);
				}
			);
		}
	);
}

///Get the number of nanoseconds per elements of the Calibration
/**	@param nbPixel : number of pixels of the tables
*/
void evaluateCalibration(size_t nbPixel){
	//Let's define size of data :
	size_t nbEvent(NB_EVENT*NB_SLICE);
	size_t nbValuePerImage(nbPixel);
	size_t nbElement(nbEvent*nbValuePerImage);
	std::vector<float> vecGain(nbPixel), vecPedestal(nbPixel);
	std::fill(vecGain.begin(), vecGain.end(), 0.02f);
	std::fill(vecPedestal.begin(), vecPedestal.end(), 40.0f);
	
	std::vector<float> vecADCSignal(nbElement), vecCalibSignal(nbElement);
	std::fill(vecADCSignal.begin(), vecADCSignal.end(), 42.0f);
	
	VecIndex vecEventIdx(nbEvent), vecPixelIdx(nbPixel);	//Init vectors of index for the computation
	std::iota(vecEventIdx.begin(), vecEventIdx.end(), 0);
	std::iota(vecPixelIdx.begin(), vecPixelIdx.end(), 0);
	
	//We have to create pointer to be able to catch them by copy without losing any time
	float * tabADC = vecADCSignal.data(), *tabCalibSignal = vecCalibSignal.data();
// 	compute_calibration(tabCalibSignal, tabADC, tabPed, tabGain, vecEventIdx, vecPixelIdx);
	
	size_t fullNbElement(nbElement);
	micro_benchmarkAutoNsPrint("evaluateCalibration 2d tranform", fullNbElement, compute_calibration, tabCalibSignal, tabADC, vecPedestal, vecGain, vecEventIdx, vecPixelIdx);
	
	// 	for(size_t i(0lu); i < 10lu; ++i){
// 		std::cout << "res["<<i<<"] = " << vecCalibSignal[i*nbValuePerImage] << std::endl;
// 	}
}

int main(int argc, char** argv){
	return micro_benchmarkParseArg(argc, argv, evaluateCalibration);
}
